--[[

$Revision: 81 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Initialization

local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes")
BossNotes = LibStub("AceAddon-3.0"):NewAddon("BossNotes",
	"AceConsole-3.0",
	"AceEvent-3.0",
	"AceTimer-3.0")

	
----------------------------------------------------------------------
-- Constants

-- Ace configuration options
local ACE_OPTS = {
	name = "Boss Notes",
	handler = BossNotes,
	type = "group",
	args = {
		toggle = {
			name = L["TOGGLE"],
			desc = L["TOGGLE_DESC"],
			order = 1,
			guiHidden = true,
			type = "execute",
			func = "ToggleBossNotesFrame"
		},
		options = {
			name = L["OPTIONS"],
			desc = L["OPTIONS_DESC"],
			order = 2,
			guiHidden = true,
			type = "execute",
			func = "ShowOptions",
		},
		instances = {
			name = L["INSTANCES"],
			desc = L["INSTANCES_DESC"],
			order = 3,
			type = "group",
			args = {
			}
		}
	}
}

-- DB defaults
local DB_DEFAULTS = {
	global = {
	}
}

-- Database version
BOSS_NOTES_DATABASE_VERSION = 1

-- Bindings
BINDING_HEADER_BOSS_NOTES = L["BOSS_NOTES"]
BINDING_NAME_BOSS_NOTES = L["TOGGLE"]


----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function BossNotes:OnInitialize()
	-- Fixup options and defaults
	local order = 1
	for _, instanceSet in ipairs(BOSS_NOTES_ENCOUNTERS) do
		ACE_OPTS.args.instances.args[instanceSet.instanceSetId] = {
			name = instanceSet.name,
			order = order,
			arg = instanceSet.instanceSetId,
			type = "toggle",
			get = "GetInstanceSet",
			set = "SetInstanceSet"
		}
		order = order + 1
		DB_DEFAULTS.global["instanceSet" .. instanceSet.instanceSetId] = instanceSet.default
	end
	
	-- Database
	self.db = LibStub("AceDB-3.0"):New("BossNotesDB")
	self.db:RegisterDefaults(DB_DEFAULTS)
	self.db.global.databaseVersion = BOSS_NOTES_DATABASE_VERSION
	
	-- Subsystems
	self:InitializeData()
	self:InitializeBossNotesFrame()
		
	-- Options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("BossNotes", ACE_OPTS, L["SLASH_COMMANDS"])
	self.options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BossNotes", L["BOSS_NOTES"])

	-- LDB
	self.dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("BossNotes", { 
		type = "data source",
		text = "General",
		icon = "Interface\\Icons\\INV_Misc_Note_06.blp",
		OnClick = function (frame, button)
			if button == "RightButton" then
				self:ShowOptions()
			else
				self:ToggleBossNotesFrame()
			end
		end,
		OnTooltipShow = function (tooltip)
			self:OnTooltipShow(tooltip)
		end
	})
end

-- Handles (re-)enabling of the addon
function BossNotes:OnEnable ()
	-- Register
	self:RegisterMessage("BOSS_NOTES_SELECTION")
	
	-- Subsystems
	self:EnableData()
	self:EnableBossNotesFrame()
end

-- Handles disabling of the addon
function BossNotes:OnDisable ()
	-- Subsystems
	self:DisableBossNotesFrame()
	self:DisableData()
end


----------------------------------------------------------------------
-- Event handlers

-- Handles the BOSS_NOTES_SELECTION event
function BossNotes:BOSS_NOTES_SELECTION (message, instance, encounter)
	self.dataObject.text = self:GetContextText(instance, encounter)
end

-- Handles the LDB show tooltip request
function BossNotes:OnTooltipShow (tooltip)
	-- Get the selection
	local instance, encounter, source = self:GetSelection()
	
	-- Format the header line
	tooltip:AddLine(string.format(L["UI_SOURCE_ON_CONTEXT"],
			source.color.r * 255, source.color.g * 255, source.color.b * 255,
			source.title, self:GetContextText(instance, encounter)))
	tooltip:AddLine(" ")
	
	-- Format the note
	if source.content then
		for line in string.gmatch(self:StripHtml(source.content), "[^\r\n]+") do
			local pos = 1
			local escapeLength = 0
			line = string.gsub(line, "%s+()(%S+)()", function (start, word, stop)
				local wordEscapeLength = self:GetEscapeLength(word)
				escapeLength = escapeLength + wordEscapeLength
				if stop - pos - escapeLength > 60 then
					pos = start
					escapeLength = wordEscapeLength
					return "\n" .. word
				end
			end)
			tooltip:AddLine(line)
		end
	else
		tooltip:AddLine(L["NO_NOTE"])
	end
end

-- Returns the length of the escapes contained in a string
function BossNotes:GetEscapeLength (s)
	local result = 0
	if string.find(s, "|") then
		for escape in string.gmatch(s, "|c%x%x%x%x%x%x%x%x") do
			result = result + string.len(escape)
		end
		for escape in string.gmatch(s, "|r") do
			result = result + string.len(escape)
		end
		for escape in string.gmatch(s, "(|H.-)|h") do
			result = result + string.len(escape)
		end
		for escape in string.gmatch(s, "|h") do
			result = result + string.len(escape)
		end
	end
	return result
end


----------------------------------------------------------------------
-- Print

-- Prints a message
function BossNotes:Print (message)
	DEFAULT_CHAT_FRAME:AddMessage("|cffe76f00" .. L["BOSS_NOTES"] .. ":|r " .. message)
end


----------------------------------------------------------------------
-- Options

-- Merges module  options
function BossNotes:MergeOptions (key, opts)
	local order = 1
	for _, opt in pairs(ACE_OPTS.args) do
		if opt.order >= order then
			order = opt.order + 1
		end
	end
	opts.order = order
	ACE_OPTS.args[key] = opts
end

-- Shows or hides the Boss Notes frame
function BossNotes:ToggleBossNotesFrame ()
	if BossNotesFrame:IsShown() then
		HideUIPanel(BossNotesFrame)
	else
		ShowUIPanel(BossNotesFrame)
	end
end

-- Shows the options panel
function BossNotes:ShowOptions ()
	InterfaceOptionsFrame_OpenToCategory(self.options)
end

-- Returns an instance set option
function BossNotes:GetInstanceSet (info, value)
	return self.db.global["instanceSet" .. info.arg]
end

-- Sets an instance set option
function BossNotes:SetInstanceSet (info, value)
	self.db.global["instanceSet" .. info.arg] = value
	self:UpdateInstanceEncounters()
	self:UpdateInstances()
end
